//
// This code is part of https://www.grapecity.com/documents-api-word/demos.
// Copyright (c) GrapeCity, Inc. All rights reserved.
//
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.IO;
using System.Linq;
using System.Drawing;
using GrapeCity.Documents.Word;
using GrapeCity.Documents.Pdf;

namespace GcWordWeb.Samples
{
    public static class Util
    {
        private static Random s_rnd = new Random((int)DateTime.Now.Ticks);
        private static object s_rnd_lock = new object();

        public static List<string> LoremIpsum(
            int numParagraphs = 5,
            int minSentences = 5,
            int maxSentences = 10,
            int minWords = 5,
            int maxWords = 30)
        {
            List<string> result = new List<string>();
            for (int p = 0; p < numParagraphs; p++)
                result.Add(LoremIpsumPar(minSentences, maxSentences, minWords, maxWords));
            return result;
        }

        // https://stackoverflow.com/questions/4286487/is-there-any-lorem-ipsum-generator-in-c
        public static string LoremIpsumPar(
            int minSentences = 5,
            int maxSentences = 10,
            int minWords = 5,
            int maxWords = 30)
        {
            var words = new[]
            {
                "lorem", "ipsum", "dolor", "sit", "amet", "consectetur", "adipiscing", "elit", "mauris", "id", "volutpat",
                "tellus", "ullamcorper", "sem", "praesent", "et", "ante", "laoreet", "lobortis", "nunc", "congue",
                "nisi", "donec", "ac", "tempus", "sed", "feugiat", "pulvinar", "pharetra", "turpis", "nonummy",
                "at", "felis", "eget", "molestie", "euismod", "non", "aliquet", "diam", "proin", "mi", "nibh",
                "massa", "tincidunt", "ut", "dolore", "magna", "aliquam", "erat",
            };
            int commaMax = 10;
            int numSentences;
            int numWords;
            lock (s_rnd_lock)
            {
                numSentences = s_rnd.Next(maxSentences - minSentences) + minSentences + 1;
                numWords = s_rnd.Next(maxWords - minWords) + minWords + 1;
            }
            List<string> result = new List<string>();
            StringBuilder par = new StringBuilder();
            for (int s = 0; s < numSentences; s++)
            {
                for (int w = 0; w < numWords; w++)
                {
                    string word;
                    lock (s_rnd_lock)
                        word = words[s_rnd.Next(words.Length)];
                    if (w == 0)
                    {
                        par.Append(char.ToUpper(word[0]));
                        par.Append(word.Substring(1));
                    }
                    else
                    {
                        lock (s_rnd_lock)
                            if (s_rnd.Next(commaMax) == 0)
                                par.Append(',');
                        par.Append(' ');
                        par.Append(word);
                    }
                }
                par.Append(". ");
            }
            return par.ToString();
        }

        // https://stackoverflow.com/questions/273313/randomize-a-listt
        public static void Shuffle<T>(this IList<T> list)
        {
            int n = list.Count;
            while (n > 1)
            {
                n--;
                int k;
                lock (s_rnd_lock)
                    k = s_rnd.Next(n + 1);
                T value = list[k];
                list[k] = list[n];
                list[n] = value;
            }
        }

        /// <summary>
        /// Adds a note to the document - a text fragment on a yellow background.
        /// Used to insert meta info into sample documents.
        /// </summary>
        /// <param name="text">The text to draw.</param>
        /// <param name="page">The page to draw on.</param>
        /// <param name="bounds">The text bounds. If null, whole page with 1" margins all around is used.</param>
        /// <returns>The actual rectangle occupied by the note.</returns>
        public static RectangleF AddNote(string text, Page page, RectangleF? bounds = null)
        {
            var g = page.Graphics;
            var tl = g.CreateTextLayout();
            var pad = g.Resolution / 8f;
            tl.DefaultFormat.Font = StandardFonts.Helvetica;
            tl.DefaultFormat.FontSize = 12;
            if (!bounds.HasValue)
            {
                var margin = g.Resolution; // 1" margins by default
                bounds = new RectangleF(margin, margin, page.Size.Width - margin * 2, page.Size.Height - margin * 2);
            }
            tl.MaxWidth = bounds.Value.Width;
            tl.MaxHeight = bounds.Value.Height;
            tl.MarginLeft = tl.MarginRight = tl.MarginTop = tl.MarginBottom = pad;
            tl.Append(text);
            tl.PerformLayout(true);

            var rect = tl.ContentRectangle;
            rect.Offset(bounds.Value.Location);
            rect.Inflate(pad, pad);

            // var qq = GrapeCity.Documents.Word.Color
            g.FillRectangle(rect, System.Drawing.Color.PaleGoldenrod);
            g.DrawRectangle(rect, System.Drawing.Color.DarkGoldenrod, 0.5f);
            g.DrawTextLayout(tl, bounds.Value.Location);

            return rect;
        }
    }
}
